﻿using System;

namespace WobbrockLib.Extensions
{
    /// <summary>
    /// Implements a set of static time-related utility functions. Any function added to this 
    /// class must be application-agnostic and have wide potential applicability, and should be
    /// of a time-related nature. Think of it as an extension to System.DateTime.
    /// </summary>
    public static class TimeEx
    {
        /// <summary>
        /// Gets the current clock time in clock-tick units (10^-7 second).
        /// </summary>
        public static long NowTicks
        {
            get
            {
                return DateTime.Now.Ticks;
            }
        }

        /// <summary>
        /// Gets the current clock time in millisecond units (10^-3 second).
        /// </summary>
        public static long NowMs
        {
            get
            {
                return DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
            }
        }

        /// <summary>
        /// Gets the current clock time in second units (10^0 second).
        /// </summary>
        public static double NowSec
        {
            get
            {
                return (double) DateTime.Now.Ticks / TimeSpan.TicksPerSecond;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static double Ticks2Sec(long ticks)
        {
            return ticks / (double) TimeSpan.TicksPerSecond;
        }

        /// <summary>
        /// 
        /// </summary>
        public static double Ticks2Sec(long ticks, int digits)
        {
            double sec = ticks / (double) TimeSpan.TicksPerSecond;
            return Math.Round(sec, digits);
        }

        /// <summary>
        /// 
        /// </summary>
        public static double Ticks2Ms(long ticks)
        {
            return ticks / (double) TimeSpan.TicksPerMillisecond;
        }

        /// <summary>
        /// 
        /// </summary>
        public static double Ticks2Ms(long ticks, int digits)
        {
            double ms = ticks / (double) TimeSpan.TicksPerMillisecond;
            return Math.Round(ms, digits);
        }

        /// <summary>
        /// 
        /// </summary>
        public static double Ticks2Min(long ticks)
        {
            return ticks / (double) TimeSpan.TicksPerMinute;
        }

        /// <summary>
        /// 
        /// </summary>
        public static double Ticks2Min(long ticks, int digits)
        {
            double min = ticks / (double) TimeSpan.TicksPerMinute;
            return Math.Round(min, digits);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sec"></param>
        /// <returns></returns>
        public static long Sec2Ticks(double sec)
        {
            return (long) (sec * TimeSpan.TicksPerSecond);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ms"></param>
        /// <returns></returns>
        public static long Ms2Ticks(double ms)
        {
            return (long) (ms * TimeSpan.TicksPerMillisecond);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="min"></param>
        /// <returns></returns>
        public static long Min2Ticks(double min)
        {
            return (long) (min * TimeSpan.TicksPerMinute);
        }

    }
}
