﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using WobbrockLib;
using WobbrockLib.Devices;
using WobbrockLib.Extensions;
using WobbrockLib.Controls;
using WobbrockLib.Optimization;
using WobbrockLib.Optimization.Genetic;
using WobbrockLib.Windows;

namespace Test
{
	public partial class MainForm : Form
	{
		private bool _isDown;
		private Point _offset;
		private LowLevelMouseHook _llmhk;
		private LowLevelKeyboardHook _llkhk;
		private RawInputMouse _rim;
		private RawInputKeyboard _rik;
		//private Thread _optimizationThread;

		public MainForm()
		{
			InitializeComponent();
			_isDown = false;

			_llmhk = new LowLevelMouseHook("Low-Level Mouse Hook");
			_llmhk.HookMouseMove += new EventHandler<MouseEventArgs>(MouseHook_MouseMove);
			_llmhk.HookMouseDown += new EventHandler<MouseEventArgs>(MouseHook_MouseDown);
			_llmhk.HookMouseUp += new EventHandler<MouseEventArgs>(MouseHook_MouseUp);
			_llmhk.HookMouseClick += new EventHandler<MouseEventArgs>(MouseHook_MouseClick);
			_llmhk.HookMouseDoubleClick += new EventHandler<MouseEventArgs>(MouseHook_MouseDoubleClick);
			_llmhk.HookMouseWheel += new EventHandler<MouseEventArgs>(MouseHook_MouseWheel);

			_llkhk = new LowLevelKeyboardHook("Low-Level Keyboard Hook");
			_llkhk.HookKeyDown += new EventHandler<KeyEventArgs>(KeyboardHook_KeyDown);
			_llkhk.HookKeyPress += new EventHandler<KeyPressEventArgs>(KeyboardHook_KeyPress);
			_llkhk.HookKeyUp += new EventHandler<KeyEventArgs>(KeyboardHook_KeyUp);

			_rim = new RawInputMouse("Raw Input Mouse");
			_rim.RawMouseMove += new EventHandler<MouseEventArgs>(RawMouse_MouseMove);
			_rim.RawMouseDown += new EventHandler<MouseEventArgs>(RawMouse_MouseDown);
			_rim.RawMouseUp += new EventHandler<MouseEventArgs>(RawMouse_MouseUp);
			_rim.RawMouseClick += new EventHandler<MouseEventArgs>(RawMouse_MouseClick);
			_rim.RawMouseDoubleClick += new EventHandler<MouseEventArgs>(RawMouse_MouseDoubleClick);
			_rim.RawMouseWheel += new EventHandler<MouseEventArgs>(RawMouse_MouseWheel);

			_rik = new RawInputKeyboard("Raw Input Keyboard");
			_rik.RawKeyDown += new EventHandler<KeyEventArgs>(RawKeyboard_KeyDown);
			_rik.RawKeyPress += new EventHandler<KeyPressEventArgs>(RawKeyboard_KeyPress);
			_rik.RawKeyUp += new EventHandler<KeyEventArgs>(RawKeyboard_KeyUp);
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			cboDirection.SelectedIndex = 0;

			Plot.Series gs0 = new Plot.Series("Test\\Series", Color.Red, Color.Black, true, true);
			gs0.AddPoint(0, 0);
			gs0.AddPoint(1, 0);
			gs0.AddPoint(2, 0);
			graph1.AddSeries(gs0);

			//Graph.Series gs1 = new Graph.Series("Test#Series", Color.Blue, Color.Green, true, true);
			//gs1.AddPoint(0, 0);
			//gs1.AddPoint(10, 20);
			//gs1.AddPoint(30, 15);
			//gs1.AddPoint(40, 0);
			//graph1.AddSeries(gs1);

			//Graph.Series gs2 = new Graph.Series("Test!Series", Color.Green, Color.Brown, true, true);
			//gs2.AddPoint(0, 0);
			//gs2.AddPoint(10, 30);
			//gs2.AddPoint(40, 10);
			//graph1.AddSeries(gs2);

			//this.Show();
			//this.Update();

			//Cursor.Position = PointToScreen(new Point(0, 0));
			//System.Threading.Thread.Sleep(1000);
			//Win32.BlockInput(true);
			//Cursor.Position = PointToScreen(new Point(ClientRectangle.Width, ClientRectangle.Height));
			//Win32.BlockInput(false);

			//_llmhk.Install(); // mouse
			//_llkhk.Install(); // keyboard
			//bool success = _rim.Register(this.Handle, true); // mouse
			bool success = _rik.Register(this.Handle, true); // hook keyboard and rik keyboard can't coexist
			//this.optLLKybdHook_CheckedChanged(sender, e);

			//keyShortcut.ShortcutChanged += new EventHandler<Keys>(OnShortcutChanged);
			//keyShortcut.Shortcut = Keys.Shift;
			//keyShortcut.DropDownStyle = ComboBoxStyle.DropDownList;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			bool success = _rim.Unregister();
			success = _rik.Unregister();
			_llmhk.Uninstall();
			_llkhk.Uninstall();
			//_optimizationThread?.Abort();
			//_optimizationThread?.Join();
		}

		/// <summary>
		/// Required for making a Raw Input device function properly.
		/// </summary>
		/// <param name="m">The Windows message received.</param>
		protected override void WndProc(ref Message m)
		{
			if (m.Msg == (int) Win32.WM.INPUT)
			{
				bool processed = _rim.ProcessRawInput(m.LParam); // mouse messages
				if (!processed)
					processed = _rik.ProcessRawInput(m.LParam); // keyboard messages
			}
			base.WndProc(ref m);
		}

		// Test WobbrockLib here...
		private void cmdTest_Click(object sender, EventArgs e)
		{
			//int i = RandomEx.Integer(0, lstdiTest.Items.Count - 1);
			//bool enabled = lstdiTest.GetItemEnabled(i);
			//lstdiTest.SetItemEnabled(i, !enabled);

			//Console.WriteLine("x=[{0},{1}], y=[{2},{3}]", graph1.XMin, graph1.XMax, graph1.YMin, graph1.YMax);

			//Console.WriteLine(ControlPanel.MouseGain);
			//bool on = ControlPanel.EnhancePointingPrecision;
			//Console.WriteLine(on);
			//ControlPanel.EnhancePointingPrecision = !on;
			//on = ControlPanel.EnhancePointingPrecision;
			//Console.WriteLine(on);

			//
			// Optimization Algorithms
			//
			//_optimizationThread = new Thread(new ThreadStart(HillClimbing));
			//_optimizationThread.IsBackground = true;
			//_optimizationThread.Start(); // triggers HillClimbing fn on new thread
			//_optimizationThread.Join(); // would block here until hill climbing ends
			//Console.WriteLine("Here");

			//RealSolution start = new RealSolution(0.01, 1.00);

			// Steepest Ascent Hill Climbing
			//HillClimber climber = new HillClimber();
			//climber.SatisficingCost = Math.PI - Math.Round(Math.PI, 5); ;
			//RealSolution rs = (RealSolution) climber.Optimize(start.GetRandom(), Math.PI);
			//Console.WriteLine("\nHILL CLIMBING solution {0} found with cost {1:G6} after {2:N0} moves.", rs.State, rs.GetCost(Math.PI), climber.NumMoves);

			// Shotgun Hill Climbing
			//ShotgunClimbers climbers = new ShotgunClimbers();
			//climbers.SatisficingCost = Math.PI - Math.Round(Math.PI, 5);
			//rs = (RealSolution) climbers.Optimize(start.GetRandom(), 1000, Math.PI);
			//Console.WriteLine("SHOTGUN solution {0} found with cost {1:G6} after {2:N0} moves.", rs.State, rs.GetCost(Math.PI), climbers.NumMoves);

			// Simulated Annealing
			//SimulatedAnnealer annealer = new SimulatedAnnealer();
			//annealer.SatisficingCost = Math.PI - Math.Round(Math.PI, 5); ;
			//rs = (RealSolution) annealer.Optimize(start.GetRandom(), -1, -1, 0.03, 100, Math.PI);
			//Console.WriteLine("SIMULATED ANNEALING solution {0} found with cost {1:G6} after {2:N0} moves.", rs.State, rs.GetCost(Math.PI), annealer.NumMoves);

			// Genetic Algorithm
			//Ecosystem eco = new Ecosystem();
			//List<Organism> organisms = eco.Populate(1000, new NumberSeekingOrganism(), Math.PI); // initial population
			//FitnessComparer comparer = new FitnessComparer(Math.PI);
			//NumberSeekingOrganism soln = (NumberSeekingOrganism) eco.Evolve(organisms, comparer, 1000, 0.20, 0.05, 0, Math.PI);
			//Console.WriteLine("GENETIC ALGORITHM solution {0} found with cost {1:G6} in {2:N0} generations.", soln, soln.GetFitness(), soln.Generation);

			//string goal = "martez";
			//Ecosystem eco = new Ecosystem();
			//List<Organism> organisms = eco.Populate(1000, new StringSeekingOrganism(0, 1, 10), goal); // initial population
			//FitnessComparer comparer = new FitnessComparer(goal);
			//long before = DateTime.Now.Ticks;
			//StringSeekingOrganism soln = (StringSeekingOrganism) eco.Evolve(organisms, comparer, 1000, 0.20, .05, -1, goal, null);
			//long after = DateTime.Now.Ticks;
			//Console.WriteLine("Solution '{0}' found in {1} generations with fitness {2:f2}.",
			//    soln.ToString(),
			//    soln.Generation,
			//    soln.GetFitness(goal));
			//Console.WriteLine("{0} ms", TimeEx.Ticks2Ms(after - before));
		}

		//private void HillClimbing()
		//{
		//    try
		//    {
		//        RealSpace space = new RealSpace(Math.PI, 1.0);
		//        RealSolution solution = null;
		//        double satisficingCost = Math.PI - Math.Round(Math.PI, 5);
		//
		//        HillClimber climber = new HillClimber();
		//        RealSolution start = new RealSolution(Math.PI);
		//        //climber.SatisficingCost = satisficingCost;
		//        solution = (RealSolution) climber.Optimize(start.Random, new OptimizationCallback(HillClimbingProgress));
		//        Console.WriteLine("\nHILL CLIMBING solution {0} found with cost {1} after {2} moves.", solution.Value, solution.Cost, climber.NumMoves);
		//    }
		//    catch (ThreadAbortException taex)
		//    {
		//        Console.WriteLine("MainForm.HillClimbing threw ThreadAbortException: {0}", taex);
		//    }
		//    catch (Exception ex)
		//    {
		//        Console.WriteLine("MainForm.HillClimbing threw Exception: {0}", ex);
		//    }
		//    finally
		//    {
		//        _optimizationThread = null;
		//    }
		//}

		//private void HillClimbingProgress(double value)
		//{
		//    int ivalue = (int) value;
		//    if ((ivalue % 10) == 0)
		//        Console.Write("{0}, ", ivalue);
		//}

		private void ShotgunHillClimbing()
		{

		}

		private void SimulatedAnnealing()
		{

		}

		private void GeneticAlgorithm()
		{

		}

		private void transparentLayer1_Click(object sender, EventArgs e)
		{
			//MessageBox.Show("TransparentLayer clicked.");
		}

		private void button1_Click(object sender, EventArgs e)
		{
			MessageBox.Show(this, "Really we ought to be going now. Really we ought to be going now.\r\nReally we ought to be going now.\r\nReally we ought to be going now.\r\nNo, really. Now.", "Hi, Mom!", MessageBoxButtons.OKCancel, MessageBoxIcon.Hand);
			PositionableMessageBox.Show(this, new Point(this.Right, this.Top), false, "Really we ought to be going now. Really we ought to be going now.\r\nReally we ought to be going now.\r\nReally we ought to be going now.\r\nNo, really. Now. Really we ought to be going now. Really we ought to be going now.\r\nReally we ought to be going now.\r\nReally we ought to be going now.\r\nNo, really. Now.\r\nNo, really. Now.\r\nNo, really. Now.", "Hi, Mom!", MessageBoxButtons.OKCancel, MessageBoxIcon.Hand, true);
		}

		/// <summary>
		/// Paint a red circle inside our layer.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void transparentLayer1_Paint(object sender, PaintEventArgs e)
		{
			//if (!transparentLayer1.Transparent)
			//	e.Graphics.FillRectangle(Brushes.White, transparentLayer1.ClientRectangle);
			e.Graphics.FillEllipse(Brushes.Red, 20, 20, 50, 50);
		}

		private void transparentLayer1_MouseDown(object sender, MouseEventArgs e)
		{
			_isDown = true;
			_offset.X = e.X;
			_offset.Y = e.Y;
		}

		private void transparentLayer1_MouseMove(object sender, MouseEventArgs e)
		{
			if (_isDown)
			{
				transparentLayer1.Left += e.X - _offset.X;
				transparentLayer1.Top += e.Y - _offset.Y;
			}
		}

		private void transparentLayer1_MouseUp(object sender, MouseEventArgs e)
		{
			_isDown = false;
		}

		private void button2_Click(object sender, EventArgs e)
		{
			transparentLayer1.Transparent = !transparentLayer1.Transparent;
		}

		private void optLLKybdHook_CheckedChanged(object sender, EventArgs e)
		{
			if (optLLKybdHook.Checked)
			{
				_rik.Unregister();
				_llkhk.Install();
				groupBox4.Enabled = true;
				groupBox3.Enabled = false;
			}
		}

		private void optRIKeyboard_CheckedChanged(object sender, EventArgs e)
		{
			if (optRIKeyboard.Checked)
			{
				_llkhk.Uninstall();
				_rik.Register(this.Handle, true);
				groupBox4.Enabled = false;
				groupBox3.Enabled = true;
			}
		}

		private void cmdWebCam_Click(object sender, EventArgs e)
		{
			WebCamForm frm = new WebCamForm();
			frm.ShowDialog(this);
		}

		private void cmdClustering_Click(object sender, EventArgs e)
		{
			ClusterTestForm kfrm = new ClusterTestForm();
			kfrm.Show(this);
		}

		private void cmdBinomial_Click(object sender, EventArgs e)
		{
			int direction;
			switch (cboDirection.SelectedIndex)
			{
				case 0:
					direction = 0;
					break;
				case 1:
					direction = -1;
					break;
				case 2:
					direction = +1;
					break;
				default:
					throw new InvalidOperationException();
			}
			try
			{
				double pvalue = StatsEx.BinomialTest((int) numSuccesses.Value, (int) numTotal.Value, (double) numProb.Value, direction);
				lblPValue.Text = String.Format("p = {0:g11}", pvalue);
				lblPValue.ForeColor = (pvalue < .05) ? Color.Red : Color.Black;
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex);
			}
		}

		//private void tttbTrackBar_Scroll(object sender, EventArgs e)
		//{
		//	Rectangle r1 = tttbTrackBar.GetThumbRect();
		//	Rectangle r2 = tttbTrackBar.GetTroughRect();
		//	tttbTrackBar.TipOffset = new Size(r2.Right - r1.Left, 0);
		//}

		//private void tttbTrackBar_MouseDown(object sender, MouseEventArgs e)
		//{
		//	Rectangle r1 = tttbTrackBar.GetThumbRect();
		//	Rectangle r2 = tttbTrackBar.GetTroughRect();
		//	tttbTrackBar.TipOffset = new Size(r2.Right - r1.Left, 0);
		//}

		#region Low-Level Mouse Hook

		private void MouseHook_MouseWheel(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Hook: Wheel {0}", e.Delta);
			lblBtn1.Text = String.Format("Wheel: {0}", e.Delta);
		}

		private void MouseHook_MouseClick(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Hook: Clk {0}", e.Button);
			lblBtn1.Text = String.Format("Btn: Clk {0}", e.Button);
		}

		private void MouseHook_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Hook: Dbl {0}", e.Button);
			lblBtn1.Text = String.Format("Btn: Dbl {0}", e.Button);
		}

		private void MouseHook_MouseDown(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Hook: Down {0}", e.Button);
			lblBtn1.Text = String.Format("Btn: Down {0}", e.Button);
		}

		private void MouseHook_MouseUp(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Hook: Up {0}", e.Button);
			lblBtn1.Text = String.Format("Btn: Up {0}", e.Button);
		}

		private void MouseHook_MouseMove(object sender, MouseEventArgs e)
		{
			lblX1.Text = String.Format("X: {0}", e.X);
			lblY1.Text = String.Format("Y: {0}", e.Y);
		}

		#endregion

		#region Raw Input Mouse

		private void RawMouse_MouseMove(object sender, MouseEventArgs e)
		{
			lblX2.Text = String.Format("X: {0}", e.X);
			lblY2.Text = String.Format("Y: {0}", e.Y);
		}

		private void RawMouse_MouseDown(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Raw: Down {0}", e.Button);
			lblBtn2.Text = String.Format("Btn: Down {0}", e.Button);
		}

		private void RawMouse_MouseUp(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Raw: Up {0}", e.Button);
			lblBtn2.Text = String.Format("Btn: Up {0}", e.Button);
		}

		private void RawMouse_MouseClick(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Raw: Clk {0}", e.Button);
			lblBtn2.Text = String.Format("Btn: Clk {0}", e.Button);
		}

		private void RawMouse_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Raw: Dbl {0}", e.Button);
			lblBtn2.Text = String.Format("Btn: Dbl {0}", e.Button);
		}

		public void RawMouse_MouseWheel(object sender, MouseEventArgs e)
		{
			Console.WriteLine("Raw: Wheel {0}", e.Delta);
			lblBtn2.Text = String.Format("Wheel: {0}", e.Delta);
		}

		#endregion

		#region Form Mouse

		private void MainForm_MouseDown(object sender, MouseEventArgs e)
		{
			//Console.WriteLine("Form: Down {0}", e.Button);
		}

		private void MainForm_MouseUp(object sender, MouseEventArgs e)
		{
			//Console.WriteLine("Form: Up {0}", e.Button);
		}

		private void MainForm_MouseClick(object sender, MouseEventArgs e)
		{
			//Console.WriteLine("Form: Clk {0}", e.Button);
		}

		private void MainForm_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			//Console.WriteLine("Form: Dbl {0}", e.Button);
		}

		#endregion

		#region Low-Level Keyboard Hook

		private void KeyboardHook_KeyUp(object sender, KeyEventArgs e)
		{
			//lblKey1.Text = String.Format("Key: up {0}", e.KeyCode);
			//lblMods1.Text = String.Format("Mods: up {0}", e.Modifiers);
			//Console.WriteLine("Hook Up => Code: {0};  Data: {1}", e.KeyCode, e.KeyData);
		}

		private void KeyboardHook_KeyPress(object sender, KeyPressEventArgs e)
		{
			//lblKey1.Text = String.Format("Key: press {0}", e.KeyChar);
			//Console.WriteLine("Hook Press => Char: {0}", e.KeyChar);
		}

		private void KeyboardHook_KeyDown(object sender, KeyEventArgs e)
		{
			lblKey1.Text = String.Format("Key: down {0}", e.KeyCode);
			lblMods1.Text = String.Format("Mods: down {0}", e.Modifiers);
			Console.WriteLine("Hook Down => Code: {0};  Data: {1}", e.KeyCode, e.KeyData);
		}

		#endregion

		#region Raw Input Keyboard

		private void RawKeyboard_KeyDown(object sender, KeyEventArgs e)
		{
			Application.DoEvents();
			lblKeys.Text = String.Format("Only Ctrl+Shift+M down? {0}", KeyboardEx.AreOnlyKeysDown(keyShortcut.ShortcutVirtualKeys));
		}

		private void RawKeyboard_KeyUp(object sender, KeyEventArgs e)
		{
			Application.DoEvents();
			lblKeys.Text = String.Format("Only Ctrl+Shift+M down? {0}", KeyboardEx.AreOnlyKeysDown(keyShortcut.ShortcutVirtualKeys));
		}

		private void RawKeyboard_KeyPress(object sender, KeyPressEventArgs e)
		{
			//lblKey2.Text = String.Format("Key: press {0}", e.KeyChar);
			//Console.WriteLine("Raw Press => Char: {0}", e.KeyChar);
		}

		#endregion

		#region Form Keyboard

		private void MainForm_KeyDown(object sender, KeyEventArgs e)
		{
			//lblKeys.Text = String.Format("Only Ctrl+Shift+M down? {0}", KeyboardEx.IsShortcutDown(Keys.Control | Keys.Shift | Keys.M));
		}

		private void MainForm_KeyPress(object sender, KeyPressEventArgs e)
		{
			//Console.WriteLine("Form Press => Char: {0}", e.KeyChar);
		}

		private void MainForm_KeyUp(object sender, KeyEventArgs e)
		{
			//lblKeys.Text = String.Format("Only Ctrl+Shift+M down? {0}", KeyboardEx.IsShortcutDown(Keys.Control | Keys.Shift | Keys.M));
		}

		#endregion

		#region Any Keys Down?

		private void tmrKeys_Tick(object sender, EventArgs e)
		{
			//if (KeyboardEx.AnyKeysDown())
			//{
			//	lblKeys.Text = "Any keys down? [TRUE]";
			//}
			//else
			//{
			//	lblKeys.Text = "Any keys down? [FALSE]";
			//}
		}

		#endregion

		#region State Timer

		private void stateTimer_Tick(object sender, object e)
		{
			Console.WriteLine(e);
		}

		#endregion

		#region Keyboard Shortcut Picker

		private void OnShortcutChanged(object sender, Keys k)
		{
			string s = keyShortcut.ToString();
			Console.WriteLine("{0} = Base: {1}, Mod: {2}", s, keyShortcut.BaseKey, keyShortcut.ModifierKeys);
		}

		private void cmdTab_Click(object sender, EventArgs e)
		{
			keyShortcut.IncludeTab = !keyShortcut.IncludeTab;
		}

		#endregion

		#region NumericUpDown Controls

		private void enterEscNumericUpDown1_ValueChanged(object sender, EventArgs e)
		{
			//Console.WriteLine("Value changed! value={0}, text={1}", enterEscNumericUpDown1.Value, enterEscNumericUpDown1.Text);
		}

		private void evenOddNumericUpDown1_ValueChanged(object sender, EventArgs e)
		{
			//Console.WriteLine("Value changed! value={0}, text={1}", evenOddNumericUpDown1.Value, evenOddNumericUpDown1.Text);
		}

		private void enterEscNumericUpDown1_UpDownButtonClick(object sender, UpDownClickEventArgs e)
		{
			//Console.WriteLine("Clicked {0}, before={1}, after={2}", e.ButtonID, e.ValueBefore, e.ValueAfter);
		}

		private void evenOddNumericUpDown1_UpDownButtonClick(object sender, UpDownClickEventArgs e)
		{
			//Console.WriteLine("Clicked {0}, before={1}, after={2}", e.ButtonID, e.ValueBefore, e.ValueAfter);
			sottNumUpDown.ToolTipTitle = "Hi, mom!";
			sottNumUpDown.Show(String.Format("Hi, mom! {0}", e.ValueAfter), evenOddNumericUpDown1, 30000);
		}

		private void enterEscNumericUpDown1_MouseEnter(object sender, EventArgs e)
		{
			Console.WriteLine("Entered enterEscNumericUpDown1");
		}

		private void enterEscNumericUpDown1_MouseHover(object sender, EventArgs e)
		{
			Console.WriteLine("Hovered over enterEscNumericUpDown1");
		}

		private void enterEscNumericUpDown1_MouseLeave(object sender, EventArgs e)
		{
			Console.WriteLine("Left enterEscNumericUpDown1");
		}

		private void enterEscNumericUpDown1_MouseMove(object sender, EventArgs e)
		{
			Console.WriteLine("Moved over enterEscNumericUpDown1");
		}

		#endregion

		private void button3_Paint(object sender, PaintEventArgs e)
		{
			e.Graphics.FillRectangle(Brushes.Red, 5, 5, 20, 20);
		}

		private void positionableLabelButton1_Click(object sender, EventArgs e)
		{
			SizeF pct = positionableLabelButton1.GetTextPositionPercentage(ScrollBars.Both);
			Console.WriteLine("Horiz={0}, Vert={1}", pct.Width, pct.Height);

			positionableLabelButton1.SetTextPositionPercentage(ScrollBars.Vertical, 0.52f);
			pct = positionableLabelButton1.GetTextPositionPercentage(ScrollBars.Both);
			Console.WriteLine("Horiz={0}, Vert={1}", pct.Width, pct.Height);

			positionableLabelButton1.CenterText();
			pct = positionableLabelButton1.GetTextPositionPercentage(ScrollBars.Both);
			Console.WriteLine("Horiz={0}, Vert={1}", pct.Width, pct.Height);

			positionableLabelButton1.TextPosition = new Point(positionableLabelButton1.Width - 20, positionableLabelButton1.Height / 2);
			pct = positionableLabelButton1.GetTextPositionPercentage(ScrollBars.Both);
			Console.WriteLine("Horiz={0}, Vert={1}", pct.Width, pct.Height);

			positionableLabelButton1.Enabled = false;
		}
	}
}
