﻿using System;
using System.Windows.Forms;
using WobbrockLib.Devices;
using WobbrockLib.Windows;

namespace Test
{
	public partial class MenuForm : Form
	{
		private readonly RawInputMouse _rim;
		private readonly RawInputKeyboard _rik;
		private readonly UIAComMenus _uia;

		/// <summary>
		/// 
		/// </summary>
		public MenuForm()
		{
			InitializeComponent();
			this.Menu = mnuMain;

			_rim = new RawInputMouse("MenuForm Raw Input Mouse");
			_rim.RawMouseClick += new EventHandler<MouseEventArgs>(RawInputMouse_MouseClick);

			_rik = new RawInputKeyboard("MenuForm Raw Input Keyboard");
			_rik.RawKeyDown += new EventHandler<KeyEventArgs>(RawInputKeyboard_KeyDown);

			//_uiaMenus = new UIANetMenus();
			_uia = new UIAComMenus();
			_uia.MenuOpened += new EventHandler<IntPtr>(SomeWindow_MenuOpened);
			_uia.MenuClosed += new EventHandler<IntPtr>(SomeWindow_MenuClosed);
			_uia.MenuModeStarted += new EventHandler<IntPtr>(SomeWindow_MenuStarted);
			_uia.MenuModeEnded += new EventHandler<IntPtr>(SomeWindow_MenuEnded);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void RawInputMouse_MouseClick(object sender, MouseEventArgs e)
		{
			// register for menu events on the window just clicked. this
			// event is handled before the menu would appear, so we can
			// register now and then still catch the MenuOpened event.
			//_uiaMenus.UnregisterListeners();
			IntPtr hwnd = WindowEx.GetWindowAt(e.Location, true);
			if (hwnd != IntPtr.Zero)
			{
				//_uiaMenus.RegisterListener(hwnd, UIAMenus.MenuEvent.Open, UIAMenus.UITree.Subtree);
				_uia.RegisterListener(hwnd, UIAMenus.MenuEvent.Open | UIAComMenus.MenuEventEx.Start, UIAMenus.UITree.Subtree, true);
				Console.WriteLine("No. Listeners = {0}", _uia.Listeners);
				Console.WriteLine(_uia.NowListening);
			}

			// MenuOpened fails to fire for old-style pre-.NET 4.0 
			// menus, so we have to check for it directly. however,
			// this event handler comes before the menu actually
			// appears, so we have to set a timer that will check 
			// for the menu being open after a delay.
			//tmrMenu.Interval = SystemInformation.MenuShowDelay;
			//tmrMenu.Start(hwnd);
		}

		/// <summary>
		/// Check to see if an old-style pre-.NET 4.0 menu is open.
		/// </summary>
		/// <param name="sender">The sender of this event.</param>
		/// <param name="e">The arguments for this event, which is a window handle.</param>
		private void tmrMenu_Tick(object sender, object e)
		{
			//tmrMenu.Stop();
			//if (Menu32Ex.IsMenuOpen())
			//{
			//	IntPtr hMenu = Menu32Ex.GetWindowWithOpenMenu();
			//	Console.WriteLine("Menu32Ex Menu OPEN on hWnd={0}", hMenu);
			//	_uia.RegisterListener(hMenu, UIAMenus.MenuEvent.Close | UIAComMenus.MenuEventEx.End, UIAMenus.UITree.Self);
			//}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SomeWindow_MenuOpened(object sender, IntPtr e)
		{
			Console.WriteLine("Menu OPENED on hWnd={0}", e);
			_uia.RegisterListener(e, UIAMenus.MenuEvent.Close, UIAMenus.UITree.Self, true);
			//_uia.RegisterListener(e, UIAMenus.MenuEvent.Close | UIAComMenus.MenuEventEx.End, UIAMenus.UITree.Self, true);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SomeWindow_MenuStarted(object sender, IntPtr e)
		{
			Console.WriteLine("Menu STARTED on hWnd={0}", e);
			_uia.RegisterListener(e, UIAComMenus.MenuEventEx.End, UIAMenus.UITree.Self, true);
			//_uia.RegisterListener(e, UIAMenus.MenuEvent.Close | UIAComMenus.MenuEventEx.End, UIAMenus.UITree.Self, true);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SomeWindow_MenuClosed(object sender, IntPtr e)
		{
			Console.WriteLine("Menu CLOSED on hWnd={0}", e);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SomeWindow_MenuEnded(object sender, IntPtr e)
		{
			Console.WriteLine("Menu ENDED on hWnd={0}", e);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void RawInputKeyboard_KeyDown(object sender, KeyEventArgs e)
		{
			//Console.WriteLine("RawInputKeyboard_KeyDown = [{0}]", e.KeyData);
			//tmrMenu.Interval = SystemInformation.MenuShowDelay;
			//tmrMenu.Start(null);
		}

		#region Form Events

		protected override void WndProc(ref Message m)
		{
			if (m.Msg == (int) Win32.WM.INPUT)
			{
				bool processed = _rim.ProcessRawInput(m.LParam);
				if (!processed)
					_rik.ProcessRawInput(m.LParam);
			}
			else if (m.Msg == (int) Win32.WM.INITMENUPOPUP)
			{
				//Console.WriteLine("WM_INITMENUPOPUP");
				//MenuEx.GetOpenWin32Menu();
			}
			base.WndProc(ref m);
		}

		private void MenuForm_Load(object sender, EventArgs e)
		{
			IntPtr hWnd = Win32.GetForegroundWindow();
			//_uia.RegisterListener(hWnd, UIAMenus.MenuEvent.Open, UIAMenus.UITree.Self);
			_uia.RegisterListener(hWnd, UIAMenus.MenuEvent.Open | UIAComMenus.MenuEventEx.Start, UIAMenus.UITree.Subtree, true);

			_rim.Register(this.Handle, true);
			_rik.Register(this.Handle, true);
		}

		private void MenuForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			_rim.Unregister();
			_rik.Unregister();
			_uia.UnregisterListeners(true);
		}

		private void MenuForm_KeyDown(object sender, KeyEventArgs e)
		{
			Console.WriteLine("KeyDown=[{0}]", e.KeyData);
		}

		#endregion

		#region Menu Item Handlers

		private void mniExit_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void MenuItem_Click(object sender, EventArgs e)
		{
			if (sender is MenuItem)
				Console.WriteLine(((MenuItem) sender).Text);
			else if (sender is ToolStripMenuItem)
				Console.WriteLine(((ToolStripMenuItem) sender).Text);
		}

		#endregion

		#region Menu Opening Events

		private void mnuFile_Popup(object sender, EventArgs e)
		{
			tmrMenu.Start(mnuFile.Handle);
		}

		private void mnuEdit_Popup(object sender, EventArgs e)
		{
			tmrMenu.Start(mnuEdit.Handle);
		}

		private void mniMore_Popup(object sender, EventArgs e)
		{
			tmrMenu.Start(mniMore.Handle);
		}

		private void mnuView_Popup(object sender, EventArgs e)
		{
			tmrMenu.Start(mnuView.Handle);
		}

		private void mnuHelp_Popup(object sender, EventArgs e)
		{
			tmrMenu.Start(mnuHelp.Handle);
		}

		private void mnuContext_Popup(object sender, EventArgs e)
		{
			tmrMenu.Start(mnuContext.Handle);
		}

		//
		// Menu Strip-Style Menus
		//

		private void mnuFileEsp_DropDownOpened(object sender, EventArgs e)
		{
			tmrMenu.Start(null);
		}

		private void mnuEditEsp_DropDownOpened(object sender, EventArgs e)
		{
			tmrMenu.Start(null);
		}

		private void viewToolStripMenuItem_DropDownOpened(object sender, EventArgs e)
		{
			tmrMenu.Start(null);
		}

		private void mnusContext_Opened(object sender, EventArgs e)
		{
			tmrMenu.Start(null);
		}

		#endregion
	}
}
