﻿using System;
using WobbrockLib.Extensions;
using WobbrockLib.Optimization.Genetic;

namespace Test
{
    public class NumberSeekingOrganism : Organism
    {
        private double _value;

        public NumberSeekingOrganism()
            : this(0, 0.0)
        {
            // do nothing
        }

        public NumberSeekingOrganism(int generation, double value)
        {
            _generation = generation;
            _value = value;
        }

        public override Organism Copy()
        {
            return new NumberSeekingOrganism(_generation, _value);
        }

        public override object DNA
        {
            get { return _value; }
        }

        public override Organism BirthRandom(object state = null)
        {
            NumberSeekingOrganism o = new NumberSeekingOrganism(1, RandomEx.Double(-1000.0, +1000.0));

            if (state != null) // set fitness at birth
                o._fitness = o.ComputeFitness(state);

            return o;
        }

        public override Organism BirthMutated(int nMutations, object state = null)
        {
            NumberSeekingOrganism o = (NumberSeekingOrganism) this.Copy();
            o._generation = _generation + 1; // next generation
            for (int i = 0; i < nMutations; i++)
            {
                int decimals = RandomEx.Integer(0, 10);
                int digit = RandomEx.Integer(0, 9);
                int posneg = RandomEx.Boolean() ? +1 : -1;
                double delta = Math.Pow(10, -decimals) * digit * posneg;
                o._value += delta;
            }

            if (state != null) // set fitness at birth
                o._fitness = o.ComputeFitness(state);

            return o;
        }

        public override int Size
        {
            get { return this.ToString().Length; }
        }

        public override object this[int index]
        {
            get 
            { 
                return this.ToString()[index];
            }
            set
            {
                string s = this.ToString();
                char[] cs = s.ToCharArray();
                cs[index] = (char) value;
                double v;
                if (double.TryParse(new string(cs), out v))
                    _value = v;
            }
        }

        public override double ComputeFitness(object state)
        {
            double goal = (double) state;
            return -Math.Abs(goal - _value); // negate so that higher values are more fit, with 0 as perfect
        }

        public override bool IsFitEnough(object state = null)
        {
            double goal = (double) state;
            return (_value == goal);
        }

        public override bool HasConvergedWith(Organism o, object state = null)
        {
            NumberSeekingOrganism p = (NumberSeekingOrganism) o;
            double r1 = Math.Round(_value, 10);
            double r2 = Math.Round(p._value, 10);
            return (r1 == r2);
        }
    }
}
