﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WobbrockLib.Extensions;
using WobbrockLib.Optimization;

namespace Test
{
    /// <summary>
    /// Implements a class that represents a solution in a real-valued number space.
    /// </summary>
    public class RealSolution : Optimization
    {
        private readonly double _minMoveSize;
        private readonly double _maxMoveSize;
        private double _solution;

        /// <summary>
        /// Constructs an empty solution with no number value.
        /// </summary>
        public RealSolution(double minMoveSize, double maxMoveSize)
        {
            _minMoveSize = minMoveSize;
            _maxMoveSize = maxMoveSize;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="target"></param>
        /// <param name="minMoveSize"></param>
        /// <param name="maxMoveSize"></param>
        /// <param name="value"></param>
        public RealSolution(double minMoveSize, double maxMoveSize, double value)
        {
            _minMoveSize = minMoveSize;
            _maxMoveSize = maxMoveSize;
            _solution = value;
        }

        /// <summary>
        /// 
        /// </summary>
        public double MinMoveSize
        {
            get { return _minMoveSize; }
        }

        /// <summary>
        /// 
        /// </summary>
        public double MaxMoveSize
        {
            get { return _maxMoveSize; }
        }

        /// <summary>
        /// 
        /// </summary>
        public override object State
        {
            get
            {
                return _solution;
            }
        }

        /// <summary>
        /// Creates a deep copy of this solution in real number space.
        /// </summary>
        /// <returns></returns>
        public override Optimization Copy()
        {
            return new RealSolution(_minMoveSize, _maxMoveSize, _solution);
        }

        /// <summary>
        /// Gets a random move in real number space.
        /// </summary>
        public override Optimization GetRandom()
        {
            return new RealSolution(_minMoveSize, _maxMoveSize, RandomEx.Double(-1000.0, +1000.0)); // jump random
        }

        /// <summary>
        /// Gets the valid neighboring states from this solution state in real number space.
        /// </summary>
        public override List<Optimization> GetMoves()
        {
            double d = (double) _solution;
            List<Optimization> neighbors = new List<Optimization>(2);
            neighbors.Add(new RealSolution(_minMoveSize, _maxMoveSize, d - RandomEx.Double(_minMoveSize, _maxMoveSize))); // step left
            neighbors.Add(new RealSolution(_minMoveSize, _maxMoveSize, d + RandomEx.Double(_minMoveSize, _maxMoveSize))); // step right
            return neighbors;
        }

        /// <summary>
        /// Gets the cost of this solution.
        /// </summary>
        public override double GetCost(object state)
        {
            double target = (double) state;
            double d = (double) _solution;
            return Math.Abs(target - d);
        }
    }
}
