﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Drawing.Text;

namespace WobbrockLib.Windows
{
	/// <summary>
	/// Utility class that provides some functionality around Windows fonts.
	/// </summary>
	public static class FontEx
	{
		/// <summary>
		/// Check to see whether the given font name, size, and style exists on the system.
		/// </summary>
		/// <param name="name">The name of the font.</param>
		/// <param name="size">The size, in points, of the font.</param>
		/// <param name="style">The style of the font.</param>
		/// <returns>True if the specified font exists; false otherwise.</returns>
		public static bool Exists(string name, float size, FontStyle style)
		{
			// try to create the requested font
			Font font = new Font(name, size, style);

			// if it works, all our properties will match. if it doesn't work,
			// then "Microsoft Sans Serif" will be created instead.
			bool hasFont = (font.Name == name && font.SizeInPoints == size && font.Style == style);

			// clean up and return
			font.Dispose();
			return hasFont;
		}

		/// <summary>
		/// Not yet implemented.
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		/// <see cref="https://stackoverflow.com/questions/21986744/how-to-install-a-font-programmatically-c"/>
		public static bool Install(string path)
		{
			throw new NotImplementedException();
		}

		/// <summary>
		/// Not yet implemented.
		/// </summary>
		/// <param name="name"></param>
		/// <returns></returns>
		public static bool Uninstall(string name)
		{
			throw new NotImplementedException();
		}
	}
}
